MANIM_RESOURCES_FOLDER = 'resources/manim/'

# Manim core
MANDATORY_CONFIG_PARAMETER = 1
OPTIONAL_CONFIG_PARAMETER = 0
# I obtained manim dimensions from here: https://docs.manim.community/en/stable/faq/general.html#what-are-the-default-measurements-for-manim-s-scene
SCENE_HEIGHT = 8 # 1080 equivalent
SCENE_WIDTH = 14 + (2 / 9) # 1920 equivalent
HALF_SCENE_WIDTH = SCENE_WIDTH / 2
HALF_SCENE_HEIGHT = SCENE_HEIGHT / 2
LEFT_MARGIN = -HALF_SCENE_WIDTH
UP_MARGIN = HALF_SCENE_HEIGHT

